import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.InsnList;

import ch.usi.dag.disl.exception.StaticContextException;
import ch.usi.dag.disl.staticcontext.AbstractStaticContext;

public class BasicBlockID extends AbstractStaticContext {

	public String getID() {

		// validate basic block - only one end
		if (staticContextData.getRegionEnds().size() != 1) {
			throw new StaticContextException("Invalid basic block");
		}

		// get starting and ending instruction from marker
		AbstractInsnNode startInsn = staticContextData.getRegionStart();
		AbstractInsnNode endInsn = staticContextData.getRegionEnds().get(0);
		// traverse entire method code and calculate instruction index
		int bbStart = -1;
		int bbLength = 0;
		boolean startFound = false;
		boolean endFound = false;
		InsnList code = staticContextData.getMethodNode().instructions;
		for (AbstractInsnNode insn = code.getFirst(); insn != null; insn = insn
				.getNext()) {
			// increase block start index until start instruction found
			if (!startFound) {
				if (insn.getOpcode() != -1) {
					++bbStart;
				}
				startFound = (insn == startInsn);
			}

			if (startFound) {
				// count instructions and exit when end instruction found
				if (insn.getOpcode() != -1) {
					++bbLength;
				}
				if (insn == endInsn) {
					endFound = true;
					break;
				}
			}
		}

		// validate basic block id
		if (!endFound) {
			throw new StaticContextException("Cannot find basic block");
		}

		// construct and return basic block id
		return bbStart + "(" + bbLength + ")";
	}
}